import argparse
import textwrap


parser = argparse.ArgumentParser(
    prog='python mlplatform_client',
    formatter_class=argparse.RawDescriptionHelpFormatter,
    description=textwrap.dedent('''
ML Platform Client 

For show all methods please use 'help' for example:

from mlplatform_client import MLPlatform
help(MLPlatform)  

First steps:  
    
1. Сreate an account in VK Cloud https://mcs.mail.ru/
2. Create you personal refresh token from UI (ML Platform -> Tokens) and save it (for example to .env file)
3. Create client for next usage
-----------------------------------------------------------------------
import os
from mlplatform_client import MLPlatform


mlp_client = MLPlatform(refresh_token=os.getenv("REFRESH_TOKEN"))
-----------------------------------------------------------------------
4. Get default manifest
-----------------------------------------------------------------------
default_manifest = mlp_client.get_default_manifest(cluster_id="cluster_id", job_name="job_name")
-----------------------------------------------------------------------

5. Get list of clusters and choose to which to submit spark job
-----------------------------------------------------------------------
clusters = mlp_client.get_clusters()

CLUSTER_ID = clusters[0].id
BUCKET_NAME = clusters[0].s3_bucket_name
-----------------------------------------------------------------------

6. Configure default manifest for your task
-----------------------------------------------------------------------
# OPTIONAL set main executable file
# when submitting manifest with python code skip this step
default_manifest.main_app_file = f"s3a://{BUCKET_NAME}/spark-files/new_main_file.py"

default_manifest.set_driver_settings(
    {"cores": 3, "env": [{"name": "NEW_ENV", "value": "NEW_ENV_VALUE"}]}
)
default_manifest.set_spark_conf(
    {"spark.driver.extraJavaOptions": "-Divy.cache.dir=/tmp -Divy.home=/tmp"}
)
# replace dependencies with new values
default_manifest.set_jars([f"s3a://{BUCKET_NAME}/test.jar"])
default_manifest.set_files([f"s3a://{BUCKET_NAME}/dataset.csv"])
# append new values to dependencies
default_mainfest.add_pyfiles([f"s3a://{BUCKET_NAME}/python_file.py"])
# add new enviroment variables
default_manifest.add_driver_env([{"name": "MY_DRIVER_ENV", "value": "my_env_value"}])
default_manifest.add_executor_env([{"name": "MY_EXECUTOR_ENV", "value": "my_env_value"}])
default_manifest.add_driver_env_from([{"secretRef": {"name": "my-secret"}}])
default_manifest.add_executor_env_from([{"secretRef": {"name": "my-secret"}}])
# set the restart policy
default_manifest.set_restart_policy(
    restart_type=K8sSparkJobRestartPolicyType.ON_FAILURE,
    on_failure_retries=3,
    on_failure_retry_interval=20,
    on_submission_failure_retries=5,
    on_submission_failure_retry_interval=20,
)
-----------------------------------------------------------------------

7. Submit configured manifest to spark operator cluster
-----------------------------------------------------------------------
mlp_client.spark_submit_job(cluster_id=CLUSTER_ID, manifest=default_manifest)
-----------------------------------------------------------------------

8. [Optional] Submit python code to spark operator cluster
-----------------------------------------------------------------------
mlp_client.spark_submit_job(cluster_id=CLUSTER_ID, manifest=default_manifest, pycode_file_path="python_code.py")
-----------------------------------------------------------------------

9. [Optional] Example submit manifest from yaml file
-----------------------------------------------------------------------
my_manifest = MLPlatform.get_mainfest_from_yaml_file(yaml_file_path="manifest.yaml")
mlp_client.spark_submit_job(cluster_id=CLUSTER_ID, manifest=my_manifest)
-----------------------------------------------------------------------

10. [Optional] Example save manifest to yaml file
-----------------------------------------------------------------------
my_manifest = MLPlatform.get_mainfest_from_yaml_file(yaml_file_path="manifest.yaml")
my_manifest.main_app_file = "local:///opt/spark/examples/src/main/python/pi.py"
my_manifest.save_yaml("manifest2.yml")
-----------------------------------------------------------------------

11. Example execute Scala jar 
-----------------------------------------------------------------------
default_manifest.spec_type = "Scala"
default_manifest.main_class = "org.apache.spark.examples.SparkPi"
default_manifest.main_app_file = "local:///opt/spark/examples/jars/spark-examples_2.12-3.4.1.jar"
mlp_client.spark_submit_job(cluster_id=CLUSTER_ID, manifest=default_manifest)
-----------------------------------------------------------------------

12. Example execute Java file from s3 
-----------------------------------------------------------------------
default_manifest.spec_type = "Java"
default_manifest.main_class = "org.apache.spark.examples.SparkPi"
default_manifest.main_app_file = f"s3a://{BUCKET_NAME}/spark-files/sparkPi.java"
mlp_client.spark_submit_job(cluster_id=CLUSTER_ID, manifest=default_manifest)
-----------------------------------------------------------------------

13. [Optional] If you are want register another user for you project, create register token (example below)
-----------------------------------------------------------------------
user_register_token = mlp_client.create_register_token(
    client_name="user_client",
    access_ttl="1h",
    refresh_ttl="2h",
)
-----------------------------------------------------------------------
`client_name` - Token name. You will see it in UI after creation.
`access_ttl` - User access token time to live. After this duration token will expire.
`refresh_ttl` - User refresh token time to live. After this duration token will expire.

By using this register token user can get personal refresh token
-----------------------------------------------------------------------
# User side
from mlplatform_client import MLPlatform

user_client = MLPlatform()
user_refresh_token = user_client.create_refresh_token(user_register_token)
-----------------------------------------------------------------------

User must remember these refresh tokens for future usage and keep it secret.
-----------------------------------------------------------------------
# User side
import os
from mlplatform_client import MLPlatform

user_client = MLPlatform(refresh_token=os.getenv("REFRESH_TOKEN"))
-----------------------------------------------------------------------

14. Administrator and user can get information about their used token.
-----------------------------------------------------------------------
print(f"Admin token info: {admin_client.who_am_i()}")
print(f"User token info: {user_client.who_am_i()}")
-----------------------------------------------------------------------

Administrator can obtain information about all project tokens.
-----------------------------------------------------------------------
print(f"Project tokens info: {admin_client.get_project_tokens_info()}")
-----------------------------------------------------------------------

15. For revoke any token you can delete it from UI (ML Platform -> Tokens).

16. For wake up cluster after pause.
-----------------------------------------------------------------------
mlp_client.wake_up_cluster(cluster_id=CLUSTER_ID)
-----------------------------------------------------------------------

17. Set cluster suspend or delete timeout(WARNING delete timeout will permanently DELETE k8s cluster with all its information).
-----------------------------------------------------------------------
mlp_client.set_cluster_suspend_time(cluster_id=CLUSTER_ID, minutes=20)
mlp_client.set_cluster_delete_time(cluster_id=CLUSTER_ID, minutes=500)
-----------------------------------------------------------------------

18. Remove cluster suspend or delete timeout
-----------------------------------------------------------------------
mlp_client.remove_cluster_suspend_time(cluster_id=CLUSTER_ID)
mlp_client.remove_cluster_delete_time(cluster_id=CLUSTER_ID)
-----------------------------------------------------------------------

19. For show detail or all information about job.
-----------------------------------------------------------------------
job = mlp.spark_job_info(cluster_id=CLUSTER_ID, job_name=JOB_NAME)

#show detail information
print(job)

#show all information
print(job.full_info)
-----------------------------------------------------------------------

20. [Optional] If you want to describe spark application
-----------------------------------------------------------------------
spark_app = mlp_client.describe_spark_job(cluster_id=CLUSTER_ID, job_name=JOB_NAME)
print(spark_app)
-----------------------------------------------------------------------

21. [Optional] If you want to work with k8s secrets or configmaps directly
-----------------------------------------------------------------------
# create/get/list/delete secret in default namespace
secret = mlp_client.create_secret_from_yaml(cluster_id=CLUSTER_ID, secret_yaml_path="path/to/secret_manifest.yaml")
secret = mlp_client.get_secret_detail(cluster_id=CLUSTER_ID, secret_name=SECRET_NAME)
secrets_list = mlp_client.list_secrets(cluster_id=CLUSTER_ID)
mlp_client.delete_secret(cluster_id=CLUSTER_ID, secret_name=SECRET_NAME)

# create/get/list/delete secret in namespace=my-namespace
secret = mlp_client.create_secret_from_yaml(cluster_id=CLUSTER_ID, secret_yaml_path="path/to/secret_manifest.yaml", namespace='my-namespace')
secret = mlp_client.get_secret_detail(cluster_id=CLUSTER_ID, secret_name=SECRET_NAME, namespace='my-namespace')
secrets_list = mlp_client.list_secrets(cluster_id=CLUSTER_ID, namespace='my-namespace')
mlp_client.delete_secret(cluster_id=CLUSTER_ID, secret_name=SECRET_NAME, namespace='my-namespace')

# create/get/list/delete configmap
configmap = mlp_client.create_configmap_from_yaml(cluster_id=CLUSTER_ID, configmap_yaml_path="path/to/configmap_manifest.yaml")
configmap = mlp_client.get_configmap_detail(cluster_id=CLUSTER_ID, configmap_name=SECRET_NAME, namespace='my-namespace')
configmaps_list = mlp_client.list_configmaps(cluster_id=CLUSTER_ID)
mlp_client.delete_configmap(cluster_id=CLUSTER_ID, configmap_name=SECRET_NAME, namespace='my-namespace')

print(secret)
print(secrets_list)
print(configmap)
print(configmaps_list)
-----------------------------------------------------------------------

22. [Optional] Show spark ui url
job = mlp.spark_job_info(cluster_id=CLUSTER_ID, job_name=JOB_NAME)
print(job.ui_url)
-----------------------------------------------------------------------

23.  Create Jupiter Hub (only for Administrator)
from mlplatform_client.serializers.gateway import MLPlatformVolumeIn, MLPlatformNetworkIn, VolumeType, AvailabilityZone


flavors = mlp.get_flavors() 
print(flavors)
#  Then choose id flavor what you want. For example 'id': '366704e5-2315-4164-bb27-c353dcc2cbbb', contain 'ram': 4096 and 'vcpus': 4
networks = mlp.get_internal_networks()
print(networks)
#  Then choose id network what you want.

jh = mlp.create_jupiter_hub(
    instance_name="jh-test",
    jh_admin_name="admin",
    jh_admin_password="Admin123@",
    flavor="366704e5-2315-4164-bb27-c353dcc2cbbb",
    s3fs_bucket="bucket-3c9c282e08a28a",  # Optional 
    volumes=[MLPlatformVolumeIn(
        size=50,
        volume_type=VolumeType.ceph_ssd,
        availability_zone=AvailabilityZone.GZ1,
    )],
    networks=MLPlatformNetworkIn(
        network_id="12345",
    ),
)
print(jh)
-----------------------------------------------------------------------

24. Get instances info (JupiterHub/MLFlow/Deploy) (only for Administrator)

all_instances_info = mlp.get_all_instances_info()
instance_info = mlp.get_instance_info(instance_id="12345")
-----------------------------------------------------------------------

25. Delete instance by id (JupiterHub/MLFlow/Deploy) (only for Administrator)

mlp.delete_instance(instance_id="1b95a730-80d0-4ded-8e0e-fe5d555ff328")
-----------------------------------------------------------------------

26. Get flavors for instances (JupiterHub/MLFlow/Deploy) (only for Administrator)

flavors = mlp.get_flavors() 
-----------------------------------------------------------------------

27. Attach S3 bucket for exist Jupiter Hub (only for Administrator)

mlp.attach_s3_bucket_to_jh(jh_id="12345", s3fs_bucket="bucket-3c9c282e08a28a") 
-----------------------------------------------------------------------

28. Create MLFlow for exist Jupiter Hub (only for Administrator)

from mlplatform_client.serializers.gateway import MLPlatformVolumeIn, MLPlatformNetworkIn, VolumeType, AvailabilityZone


mlflow = mlp.create_mlflow(
    instance_name="mlflow-test",
    jh_id="12345",
    flavor="366704e5-2315-4164-bb27-c353dcc2cbbb",
    volumes=[MLPlatformVolumeIn(
        size=50,
        volume_type=VolumeType.ceph_ssd,
        availability_zone=AvailabilityZone.GZ1,
    )],
    networks=MLPlatformNetworkIn(
        network_id="12345",
    ),
    is_mlflow_demo_mode=True,
)
print(mlflow)
-----------------------------------------------------------------------

29. Create Deploy for exists MLFlow (only for Administrator)

from mlplatform_client.serializers.gateway import MLPlatformVolumeIn, MLPlatformNetworkIn, VolumeType, AvailabilityZone


deploy = mlp.create_deploy(
    instance_name="deploy-test",
    mlflow_id="12345",
    flavor="366704e5-2315-4164-bb27-c353dcc2cbbb",
    volumes=[MLPlatformVolumeIn(
        size=50,
        volume_type=VolumeType.ceph_ssd,
        availability_zone=AvailabilityZone.GZ1,
    )],
    networks=MLPlatformNetworkIn(
        network_id="12345",
    ),
    is_mlflow_demo_mode=True,
)
print(deploy)
-----------------------------------------------------------------------

30. Get internal networks (only for Administrator)

networks = mlp.get_internal_networks()
print(networks)
-----------------------------------------------------------------------

31. Get external networks (only for Administrator)

networks = mlp.get_external_networks()
print(networks)
-----------------------------------------------------------------------

32. Create remote model deployment

deployment = mlp.create_deployment(
    deploy_server_name="deploy_server_name", deployment_name="model_deployment_name", 
    model_uri="mlflow-artifacts:/2/bc303eb8e735422c9e8ab450e6499bf3/artifacts/model", 
    username="mlflow_or_jh_admin_username", password="mlflow_or_jh_password", 
    model_username="admin", model_password="Password123!"
)
print(deployment)
-----------------------------------------------------------------------

33. Get remote model deployment

deployment = mlp.get_deployment(
    deploy_server_name="deploy_server_name", deployment_name="model_deployment_name",
    username="mlflow_or_jh_admin_username", password="mlflow_or_jh_password",
)
print(deployment)
-----------------------------------------------------------------------

34. List remote model deployments

deployments = mlp.list_deployments(
    deploy_server_name="deploy_server_name",
    username="mlflow_or_jh_admin_username", password="mlflow_or_jh_password",
)
print(deployments)
-----------------------------------------------------------------------

35. Delete remote model deployment

mlp.delete_deployment(
    deploy_server_name="deploy_server_name", deployment_name="model_deployment_name",
    username="mlflow_or_jh_admin_username", password="mlflow_or_jh_password",
)
-----------------------------------------------------------------------

36. Use remote model for inference by predict method

data = {
    "inputs": 
        [
            [0.045341,  0.050680,  0.060618,  0.031065,  0.028702, -0.047347, -0.054446, 0.071210,  0.133597, 0.135612],
            [0.075341,  0.010680,  0.030618,  0.011065,  0.098702, -0.007347, -0.014446, 0.071210,  0.093597, 0.115612]
        ]
}

# model_username and model_password are different from JupyterHub or MLflow username or password
# Use same model_username and model_password used to create deployment
result = mlp.predict_model(
    data, deploy_server_name="deploy_server_name", deployment_name="model_deployment_name",
    model_username="admin", model_password="Password123!",
)

print(result)
-----------------------------------------------------------------------

37. Ping remote model for health check

result = mlp.ping_model(
    deploy_server_name="deploy_server_name", deployment_name="model_deployment_name",
    model_username="admin", model_password="Password123!",
)

print(result) # should be new line string e.g "\n"

'''))
args = parser.parse_args()
