from typing import Dict, Any
from mlplatform_client.clients import BaseClient
from mlplatform_client.serializers.settings import ClientSettings
from mlplatform_client.serializers.deployment import UserAuth


class DeployModelClient(BaseClient):
    """Client for remote interaction with deployed models"""

    def __init__(self, deploy_server_domain: str, model_name: str, model_user_auth: UserAuth):
        super().__init__(ClientSettings(host=f"https://{deploy_server_domain}/{model_name}"))
        self.__auth_str = model_user_auth.get_basicauth()

    def _auth_header(self):
        """Get headers"""
        return {"Authorization": f"Basic {self.__auth_str}"}

    def ping(self) -> str:
        """Ping deploy model

        Returns:
            (str): string containing new line symbol e.g "\n"
        """
        return self._perform_request("get", self.get_path("ping"), headers=self._auth_header())

    def predict(self, payload: Dict) -> Dict[str, Any]:
        """Sends data for remote inference

        Args:
            payload (Dict): model input data for inference

        Return:
            Dict[str, Any]: inference result, e.g. {"predictions": [161.95369430265362, 159.0451149799379]}
        """
        return self._perform_request("post", self.get_path("invocations"), json=payload, headers=self._auth_header())
