#  Copyright (c) 2023 VK Group
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#     http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
#  implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
from typing import Optional, Any
from requests import Response


class MLPlatformException(Exception):
    """Base class for all exceptions in application"""
    default_msg = "Unknown error"

    def __init__(self, message=None, extra_info=None):
        self.message = message or self.default_msg
        if extra_info:
            self.message = f"{self.message}. Info: {self.strip_extra_info(extra_info)}"

    def __str__(self):
        return self.message

    def __repr__(self):
        return self.message

    @staticmethod
    def strip_extra_info(extra_info: str) -> str:
        pos = extra_info.find("additional info: ")
        if pos > 0:
            extra_info = extra_info[pos + len(r"additional info: {\"message\":\""):].strip().rstrip("\"}\\")
        return extra_info


class BaseClientError(MLPlatformException):
    def __init__(self, response: Optional[Response], content: Any = None):
        super().__init__(response, content)


class NotFoundException(MLPlatformException):
    default_msg = "Not Found"


class UnauthorizedException(MLPlatformException):
    default_msg = "Unauthorized"


class ForbiddenException(MLPlatformException):
    default_msg = "Forbidden"


class WrongTokenException(MLPlatformException):
    default_msg = "Wrong token passed"


class UnprocessableEntityException(MLPlatformException):
    default_msg = "Unprocessable Entity"


class ConnectionException(MLPlatformException):
    default_msg = "Connection error"


class UnavailableForLegalReasonException(MLPlatformException):
    default_msg = "Unavailable For Legal Reason"


class ConflictException(MLPlatformException):
    default_msg = "The request could not be completed due to a conflict with the current state of the resource"


class BadGatewayException(MLPlatformException):
    default_msg = "Remote server is unavailable right now due to 502 BadGateway"
