#  Copyright (c) 2023 VK Group
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#     http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
#  implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
from typing import Any, Union
from typing_extensions import Literal

from pydantic import BaseModel, ConfigDict


class BaseSerializerModel(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    def not_null_values(self):
        return self.model_dump(exclude_none=True)

    def model_dump(
        self,
        *args,
        mode: Union[Literal["json", "python"], str] = "json",
        **kwargs,
    ) -> dict[str, Any]:
        return super().model_dump(*args, mode=mode, **kwargs)

    def __repr__(self):
        return self.__str__()


class BaseAuditModel(BaseSerializerModel):

    def get_id(self) -> str:
        return getattr(self, "id")

    def get_name(self) -> str:
        return getattr(self, "name")

    def get_metadata(self) -> dict:
        return {
            k: v for k, v
            in self.not_null_values().items()
            if isinstance(v, str)
        }
