#  Copyright (c) 2023 VK Group
#  #
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#  #
#     http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
#  implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
from typing import Optional

from pydantic import Field

from mlplatform_client.serializers import BaseSerializerModel


class NovaBaseObj(BaseSerializerModel):
    id: str


class GPUInfo(BaseSerializerModel):
    model: Optional[str] = None


class FullGPUInfo(GPUInfo):
    name: Optional[str] = None
    memory: Optional[str] = None
    total_cards: Optional[int] = None


class ExtraSpecifications(BaseSerializerModel):
    cpu_sockets: Optional[int] = Field(default=None, alias="hw:cpu_sockets")
    cpu_type: Optional[str] = Field(default=None, alias="mcs:cpu_type")
    agg_gpu: Optional[bool] = Field(default=None, alias="agg_gpu")
    filter_gpu: Optional[bool] = Field(default=None, alias="filter:gpu")
    availability_zone: Optional[str] = Field(default=None, alias="availability_zone")
    gpu_info: str = Field(alias="pci_passthrough:alias", default='')

    def __str__(self):
        attrs = vars(self)
        items = ''.join("\n    %s: %s" % items for items in attrs.items())
        return f'\n{items[1:]}'

    def parsed_gpu_info(self) -> FullGPUInfo:
        raw_name = self.gpu_info.split("-")
        raw_cards_num = self.gpu_info.split(":")
        name = None
        model = None
        total_cards = None
        if self.gpu_info:
            if len(raw_name) == 2:
                name = raw_name[0]
                model = raw_name[-1]
            if len(raw_cards_num) == 2:
                total_cards = raw_cards_num[-1]

        return FullGPUInfo(
            name=name,
            model=model,
            total_cards=total_cards
        )


class NovaFlavor(NovaBaseObj):
    name: str
    ram: int
    vcpus: int
    extra_specs: Optional[ExtraSpecifications] = None

    def __str__(self):
        attrs = vars(self)
        items = ''.join("\n%s: %s" % item for item in attrs.items())
        return f'\n---------------\nFlavor Info:\n---------------{items}'

    def parsed_gpu_name_info(self) -> GPUInfo:
        raw_name = self.name.split("x")
        model = None
        if len(raw_name) == 2:
            model = raw_name[0]

        return GPUInfo(
            model=model
        )
